/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef flxpeval_H
#define flxpeval_H

#include "flxrandom.h"

class FlxPeval_term {
  private:
    int M;
    int *arr;
    #if FLX_DEBUG
      void M_check(int rhsM) const;
    #endif
    const tdouble moment_StdN(const int nth) const;
  public:
    /**
    * @brief a term of a polynom x_1^a*x_2^b*...
    * M: order
    * will assign the polynom x_p^degree_of_p
    * p starts counting from 1 ... to M
    */
    FlxPeval_term(int M, int p = 1, int degree_of_p = 0);
    /**
    * @brief Multiplication: lhs*rhs
    */
    FlxPeval_term(const FlxPeval_term& lhs, const FlxPeval_term& rhs);
    ~FlxPeval_term() { delete [] arr; }
    FlxPeval_term( const FlxPeval_term& fpt );
    FlxPeval_term& operator=( const FlxPeval_term& fpt );

    const int get_dim() const { return M; }
    /**
    * @brief calculates the expectation of the Term
    */
    const tdouble calc_Expectation() const;
    
    const bool operator<(const FlxPeval_term& rhs) const
    {
      #if FLX_DEBUG
	M_check(rhs.M);
      #endif
      for (register int i=0; i < M; ++i) {
	if (arr[i] == rhs.arr[i]) continue;
	else if (arr[i] < rhs.arr[i]) return true;
	else return false;
      }
      return false;
    }
    const bool operator>(const FlxPeval_term& rhs) const;
    const bool operator==(const FlxPeval_term& rhs) const;
    
    friend std::ostream& operator<<(std::ostream& os, const FlxPeval_term& val);
};

std::ostream& operator<<(std::ostream& os, const FlxPeval_term& val);

class FlxPeval {
  private:
    typedef std::map <FlxPeval_term, tdouble> flxtlmap;
    int M;
    flxtlmap tl;
  public:
    FlxPeval(int M, bool isone=false);
    /**
    * @brief Multiplication: lhs*rhs
    */
    FlxPeval(const FlxPeval& lhs, const FlxPeval& rhs);
    
    /**
    * @brief returns the coefficient in front of a term
    */
    tdouble get_coeff(FlxPeval_term& fpt);
    /**
    * @brief calculates the expectation of the Polynomial
    */
    const tdouble calc_Expectation() const;
    
    FlxPeval& add_term(const FlxPeval_term& tp, const tdouble& tv);
    FlxPeval& operator+=(const std::pair<FlxPeval_term, tdouble>& pv);
    FlxPeval& operator+=(const FlxPeval& rhs);
    FlxPeval& operator*=(const tdouble& rhs);
    
    friend std::ostream& operator<<(std::ostream& os, const FlxPeval& val);
};

std::ostream& operator<<(std::ostream& os, const FlxPeval& val);

#endif // flxpeval_H
